@@ -24,7 +24,7 @@ int csum_length=2; /* initial value */
 #define CSUM_CHUNK 64
 
 int checksum_seed = 0;
-extern int remote_version;
+extern int protocol_version;
 
 /*
   a simple 32 bit checksum that can be upadted from either end
@@ -74,7 +74,13 @@ void get_checksum2(char *buf,int len,char *sum)
 	for(i = 0; i + CSUM_CHUNK <= len; i += CSUM_CHUNK) {
 		mdfour_update(&m, (uchar *)(buf1+i), CSUM_CHUNK);
 	}
-	if (len - i > 0) {
+	/*
+	 * Prior to version 27 an incorrect MD4 checksum was computed
+	 * by failing to call mdfour_tail() for block sizes that
+	 * are multiples of 64.  This is fixed by calling mdfour_update()
+	 * even when there are no more bytes.
+	 */
+	if (len - i > 0 || protocol_version >= 27) {
 		mdfour_update(&m, (uchar *)(buf1+i), (len-i));
 	}
 	
@@ -105,8 +111,16 @@ void file_checksum(char *fname,char *sum,OFF_T size)
 		mdfour_update(&m, (uchar *)tmpchunk, CSUM_CHUNK);
 	}
 
+	/*
+	 * Prior to version 27 an incorrect MD4 checksum was computed
+	 * by failing to call mdfour_tail() for block sizes that
+	 * are multiples of 64.  This is fixed by calling mdfour_update()
+	 * even when there are no more bytes.
+	 */
 	if (len - i > 0) {
 		memcpy(tmpchunk, map_ptr(buf,i,len-i), len-i);
+	}
+	if (len - i > 0 || protocol_version >= 27) {
 		mdfour_update(&m, (uchar *)tmpchunk, (len-i));
 	}
 
@@ -117,16 +131,6 @@ void file_checksum(char *fname,char *sum,OFF_T size)
 }
 
 
-void checksum_init(void)
-{
-  if (remote_version >= 14)
-    csum_length = 2; /* adaptive */
-  else
-    csum_length = SUM_LENGTH;
-}
-
-
-
 static int sumresidue;
 static char sumrbuf[CSUM_CHUNK];
 static struct mdfour md;
@@ -180,7 +184,7 @@ void sum_update(char *p, int len)
 
 void sum_end(char *sum)
 {
-	if (sumresidue) {
+	if (sumresidue || protocol_version >= 27) {
 		mdfour_update(&md, (uchar *)sumrbuf, sumresidue);
 	}
 
