@@ -1,18 +1,18 @@
-/* 
+/*
    Copyright (C) Andrew Tridgell 1996
    Copyright (C) Paul Mackerras 1996
    Copyright (C) 2001, 2002 by Martin Pool <mbp@samba.org>
-   
+
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
@@ -41,6 +41,8 @@ extern int always_checksum;
 extern int cvs_exclude;
 
 extern int recurse;
+extern char *files_from;
+extern int filesfrom_fd;
 
 extern int one_file_system;
 extern int make_backups;
@@ -52,25 +54,29 @@ extern int preserve_uid;
 extern int preserve_gid;
 extern int preserve_times;
 extern int relative_paths;
+extern int implied_dirs;
 extern int copy_links;
 extern int copy_unsafe_links;
-extern int remote_version;
-extern int io_error;
+extern int protocol_version;
 extern int sanitize_paths;
 
 extern int read_batch;
 extern int write_batch;
 
-static struct exclude_struct **local_exclude_list;
+extern struct exclude_struct **exclude_list;
+extern struct exclude_struct **server_exclude_list;
+extern struct exclude_struct **local_exclude_list;
+
+int io_error;
 
 static struct file_struct null_file;
 
-static void clean_flist(struct file_list *flist, int strip_root);
+static void clean_flist(struct file_list *flist, int strip_root, int no_dups);
 
 
 static int show_filelist_p(void)
 {
-	return verbose && recurse && !am_server;
+	return verbose && (recurse || files_from) && !am_server;
 }
 
 static void start_filelist_progress(char *kind)
@@ -216,12 +222,12 @@ int readlink_stat(const char *path, STRUCT_STAT * buffer, char *linkbuf)
 	if (S_ISLNK(buffer->st_mode)) {
 		int l;
 		l = readlink((char *) path, linkbuf, MAXPATHLEN - 1);
-		if (l == -1) 
+		if (l == -1)
 			return -1;
 		linkbuf[l] = 0;
 		if (copy_unsafe_links && unsafe_symlink(linkbuf, path)) {
 			if (verbose > 1) {
-				rprintf(FINFO,"copying unsafe symlink \"%s\" -> \"%s\"\n", 
+				rprintf(FINFO,"copying unsafe symlink \"%s\" -> \"%s\"\n",
 					path, linkbuf);
 			}
 			return do_stat(path, buffer);
@@ -247,20 +253,37 @@ int link_stat(const char *path, STRUCT_STAT * buffer)
 }
 
 /*
-  This function is used to check if a file should be included/excluded
-  from the list of files based on its name and type etc
+ * This function is used to check if a file should be included/excluded
+ * from the list of files based on its name and type etc.  The value of
+ * exclude_level is set to either SERVER_EXCLUDES or ALL_EXCLUDES.
  */
-static int check_exclude_file(int f, char *fname, STRUCT_STAT * st)
+static int check_exclude_file(char *fname, int is_dir, int exclude_level)
 {
-	extern int delete_excluded;
-
-	/* f is set to -1 when calculating deletion file list */
-	if ((f == -1) && delete_excluded) {
+#if 0 /* This currently never happens, so avoid a useless compare. */
+	if (exclude_level == NO_EXCLUDES)
 		return 0;
+#endif
+	if (fname) {
+		/* never exclude '.', even if somebody does --exclude '*' */
+		if (fname[0] == '.' && !fname[1])
+			return 0;
+		/* Handle the -R version of the '.' dir. */
+		if (fname[0] == '/') {
+			int len = strlen(fname);
+			if (fname[len-1] == '.' && fname[len-2] == '/')
+				return 0;
+		}
 	}
-	if (check_exclude(fname, local_exclude_list, st)) {
+	if (server_exclude_list
+	 && check_exclude(server_exclude_list, fname, is_dir))
+		return 1;
+	if (exclude_level != ALL_EXCLUDES)
+		return 0;
+	if (exclude_list && check_exclude(exclude_list, fname, is_dir))
+		return 1;
+	if (local_exclude_list
+	 && check_exclude(local_exclude_list, fname, is_dir))
 		return 1;
-	}
 	return 0;
 }
 
@@ -306,19 +329,20 @@ static void flist_expand(struct file_list *flist)
 {
 	if (flist->count >= flist->malloced) {
 		void *new_ptr;
-		
+
 		if (flist->malloced < 1000)
 			flist->malloced += 1000;
 		else
 			flist->malloced *= 2;
 
-		if (flist->files)
+		if (flist->files) {
 			new_ptr = realloc_array(flist->files,
 						struct file_struct *,
 						flist->malloced);
-		else
+		} else {
 			new_ptr = new_array(struct file_struct *,
 					    flist->malloced);
+		}
 
 		if (verbose >= 2) {
 			rprintf(FINFO, "expand file_list to %.0f bytes, did%s move\n",
@@ -326,7 +350,7 @@ static void flist_expand(struct file_list *flist)
 				* flist->malloced,
 				(new_ptr == flist->files) ? " not" : "");
 		}
-		
+
 		flist->files = (struct file_struct **) new_ptr;
 
 		if (!flist->files)
@@ -375,7 +399,7 @@ static void send_file_entry(struct file_struct *file, int f,
 
 	for (l1 = 0;
 	     lastname[l1] && (fname[l1] == lastname[l1]) && (l1 < 255);
-	     l1++);
+	     l1++) {}
 	l2 = strlen(fname) - l1;
 
 	if (l1 > 0)
@@ -425,7 +449,7 @@ static void send_file_entry(struct file_struct *file, int f,
 
 #if SUPPORT_HARD_LINKS
 	if (preserve_hard_links && S_ISREG(file->mode)) {
-		if (remote_version < 26) {
+		if (protocol_version < 26) {
 			/* 32-bit dev_t and ino_t */
 			write_int(f, (int) file->dev);
 			write_int(f, (int) file->inode);
@@ -438,7 +462,7 @@ static void send_file_entry(struct file_struct *file, int f,
 #endif
 
 	if (always_checksum) {
-		if (remote_version < 21) {
+		if (protocol_version < 21) {
 			write_buf(f, file->sum, 2);
 		} else {
 			write_buf(f, file->sum, MD4_SUM_LENGTH);
@@ -558,7 +582,7 @@ static void receive_file_entry(struct file_struct **fptr,
 	}
 #if SUPPORT_HARD_LINKS
 	if (preserve_hard_links && S_ISREG(file->mode)) {
-		if (remote_version < 26) {
+		if (protocol_version < 26) {
 			file->dev = read_int(f);
 			file->inode = read_int(f);
 		} else {
@@ -572,7 +596,7 @@ static void receive_file_entry(struct file_struct **fptr,
 		file->sum = new_array(char, MD4_SUM_LENGTH);
 		if (!file->sum)
 			out_of_memory("md4 sum");
-		if (remote_version < 21) {
+		if (protocol_version < 21) {
 			read_buf(f, file->sum, 2);
 		} else {
 			read_buf(f, file->sum, MD4_SUM_LENGTH);
@@ -639,8 +663,8 @@ static int skip_filesystem(char *fname, STRUCT_STAT * st)
  * statting directories if we're not recursing, but this is not a very
  * important case.  Some systems may not have d_type.
  **/
-struct file_struct *make_file(int f, char *fname, struct string_area **ap,
-			      int noexcludes)
+struct file_struct *make_file(char *fname, struct string_area **ap,
+			      int exclude_level)
 {
 	struct file_struct *file;
 	STRUCT_STAT st;
@@ -662,27 +686,26 @@ struct file_struct *make_file(int f, char *fname, struct string_area **ap,
 
 	if (readlink_stat(fname, &st, linkbuf) != 0) {
 		int save_errno = errno;
-		if ((errno == ENOENT) && !noexcludes) {
-			/* either symlink pointing nowhere or file that 
+		if (errno == ENOENT && exclude_level != NO_EXCLUDES) {
+			/* either symlink pointing nowhere or file that
 			 * was removed during rsync run; see if excluded
 			 * before reporting an error */
-			memset((char *) &st, 0, sizeof(st));
-			if (check_exclude_file(f, fname, &st)) {
+			if (check_exclude_file(fname, 0, exclude_level)) {
 				/* file is excluded anyway, ignore silently */
 				return NULL;
 			}
 		}
-		io_error = 1;
-		rprintf(FERROR, "readlink %s: %s\n",
-			fname, strerror(save_errno));
+		io_error |= IOERR_GENERAL;
+		rprintf(FERROR, "readlink %s failed: %s\n",
+			full_fname(fname), strerror(save_errno));
 		return NULL;
 	}
 
-	/* we use noexcludes from backup.c */
-	if (noexcludes)
+	/* backup.c calls us with exclude_level set to NO_EXCLUDES. */
+	if (exclude_level == NO_EXCLUDES)
 		goto skip_excludes;
 
-	if (S_ISDIR(st.st_mode) && !recurse) {
+	if (S_ISDIR(st.st_mode) && !recurse && !files_from) {
 		rprintf(FINFO, "skipping directory %s\n", fname);
 		return NULL;
 	}
@@ -692,17 +715,16 @@ struct file_struct *make_file(int f, char *fname, struct string_area **ap,
 			return NULL;
 	}
 
-	if (check_exclude_file(f, fname, &st))
+	if (check_exclude_file(fname, S_ISDIR(st.st_mode) != 0, exclude_level))
 		return NULL;
 
-
 	if (lp_ignore_nonreadable(module_id) && access(fname, R_OK) != 0)
 		return NULL;
 
       skip_excludes:
 
 	if (verbose > 2)
-		rprintf(FINFO, "make_file(%d,%s)\n", f, fname);
+		rprintf(FINFO, "make_file(%s,*,%d)\n", fname, exclude_level);
 
 	file = new(struct file_struct);
 	if (!file)
@@ -780,8 +802,12 @@ void send_file_name(int f, struct file_list *flist, char *fname,
 		    int recursive, unsigned base_flags)
 {
 	struct file_struct *file;
+	extern int delete_excluded;
 
-	file = make_file(f, fname, &flist->string_area, 0);
+	/* f is set to -1 when calculating deletion file list */
+	file = make_file(fname, &flist->string_area,
+			 f == -1 && delete_excluded? SERVER_EXCLUDES
+						   : ALL_EXCLUDES);
 
 	if (!file)
 		return;
@@ -793,7 +819,7 @@ void send_file_name(int f, struct file_list *flist, char *fname,
 	if (write_batch)	/*  dw  */
 		file->flags = FLAG_DELETE;
 
-	if (strcmp(file->basename, "")) {
+	if (file->basename[0]) {
 		flist->files[flist->count++] = file;
 		send_file_entry(file, f, base_flags);
 	}
@@ -819,8 +845,9 @@ static void send_directory(int f, struct file_list *flist, char *dir)
 
 	d = opendir(dir);
 	if (!d) {
-		io_error = 1;
-		rprintf(FERROR, "opendir(%s): %s\n", dir, strerror(errno));
+		io_error |= IOERR_GENERAL;
+		rprintf(FERROR, "opendir %s failed: %s\n",
+			full_fname(dir), strerror(errno));
 		return;
 	}
 
@@ -828,10 +855,9 @@ static void send_directory(int f, struct file_list *flist, char *dir)
 	l = strlen(fname);
 	if (fname[l - 1] != '/') {
 		if (l == MAXPATHLEN - 1) {
-			io_error = 1;
-			rprintf(FERROR,
-				"skipping long-named directory %s\n",
-				fname);
+			io_error |= IOERR_GENERAL;
+			rprintf(FERROR, "skipping long-named directory: %s\n",
+				full_fname(fname));
 			closedir(d);
 			return;
 		}
@@ -845,45 +871,51 @@ static void send_directory(int f, struct file_list *flist, char *dir)
 	if (cvs_exclude) {
 		if (strlen(fname) + strlen(".cvsignore") <= MAXPATHLEN - 1) {
 			strcpy(p, ".cvsignore");
-			local_exclude_list =
-			    make_exclude_list(fname, NULL, 0, 0);
+			add_exclude_file(&exclude_list,fname,MISSING_OK,ADD_EXCLUDE);
 		} else {
-			io_error = 1;
+			io_error |= IOERR_GENERAL;
 			rprintf(FINFO,
 				"cannot cvs-exclude in long-named directory %s\n",
-				fname);
+				full_fname(fname));
 		}
 	}
 
-	for (di = readdir(d); di; di = readdir(d)) {
+	for (errno = 0, di = readdir(d); di; errno = 0, di = readdir(d)) {
 		char *dname = d_name(di);
-		if (strcmp(dname, ".") == 0 || strcmp(dname, "..") == 0)
+		if (dname[0] == '.' && (dname[1] == '\0'
+		    || (dname[1] == '.' && dname[2] == '\0')))
 			continue;
 		strlcpy(p, dname, MAXPATHLEN - l);
 		send_file_name(f, flist, fname, recurse, 0);
 	}
-
-	if (local_exclude_list) {
-		add_exclude_list("!", &local_exclude_list, 0);
+	if (errno) {
+		io_error |= IOERR_GENERAL;
+		rprintf(FERROR, "readdir(%s): (%d) %s\n",
+		    dir, errno, strerror(errno));
 	}
 
+	if (local_exclude_list)
+		free_exclude_list(&local_exclude_list); /* Zeros pointer too */
+
 	closedir(d);
 }
 
 
 /**
- *
- * I <b>think</b> f==-1 means that the list should just be built in
- * memory and not transmitted.  But who can tell? -- mbp
+ * The delete_files() function in receiver.c sets f to -1 so that we just
+ * construct the file list in memory without sending it over the wire.  It
+ * also has the side-effect of ignoring user-excludes if delete_excluded
+ * is set (so that the delete list includes user-excluded files).
  **/
 struct file_list *send_file_list(int f, int argc, char *argv[])
 {
-	int i, l;
+	int l;
 	STRUCT_STAT st;
 	char *p, *dir, *olddir;
 	char lastpath[MAXPATHLEN] = "";
 	struct file_list *flist;
 	int64 start_write;
+	int use_ff_fd = 0;
 
 	if (show_filelist_p() && f != -1)
 		start_filelist_progress("building file list");
@@ -894,23 +926,37 @@ struct file_list *send_file_list(int f, int argc, char *argv[])
 
 	if (f != -1) {
 		io_start_buffering(f);
+		if (filesfrom_fd >= 0) {
+			if (argv[0] && !push_dir(argv[0], 0)) {
+				rprintf(FERROR, "push_dir %s failed: %s\n",
+					full_fname(argv[0]), strerror(errno));
+				exit_cleanup(RERR_FILESELECT);
+			}
+			use_ff_fd = 1;
+		}
 	}
 
-	for (i = 0; i < argc; i++) {
+	while (1) {
 		char fname2[MAXPATHLEN];
 		char *fname = fname2;
 
-		strlcpy(fname, argv[i], MAXPATHLEN);
+		if (use_ff_fd) {
+			if (read_filesfrom_line(filesfrom_fd, fname) == 0)
+				break;
+			sanitize_path(fname, NULL);
+		} else {
+			if (argc-- == 0)
+				break;
+			strlcpy(fname, *argv++, MAXPATHLEN);
+			if (sanitize_paths)
+				sanitize_path(fname, NULL);
+		}
 
 		l = strlen(fname);
-		if (l != 1 && fname[l - 1] == '/') {
-			if ((l == 2) && (fname[0] == '.')) {
-				/*  Turn ./ into just . rather than ./.
-				   This was put in to avoid a problem with
-				   rsync -aR --delete from ./
-				   The send_file_name() below of ./ was
-				   mysteriously preventing deletes */
-				fname[1] = 0;
+		if (fname[l - 1] == '/') {
+			if (l == 2 && fname[0] == '.') {
+				/* Turn "./" into just "." rather than "./." */
+				fname[1] = '\0';
 			} else {
 				strlcat(fname, ".", MAXPATHLEN);
 			}
@@ -918,14 +964,14 @@ struct file_list *send_file_list(int f, int argc, char *argv[])
 
 		if (link_stat(fname, &st) != 0) {
 			if (f != -1) {
-				io_error = 1;
-				rprintf(FERROR, "link_stat %s : %s\n",
-					fname, strerror(errno));
+				io_error |= IOERR_GENERAL;
+				rprintf(FERROR, "link_stat %s failed: %s\n",
+					full_fname(fname), strerror(errno));
 			}
 			continue;
 		}
 
-		if (S_ISDIR(st.st_mode) && !recurse) {
+		if (S_ISDIR(st.st_mode) && !recurse && !files_from) {
 			rprintf(FINFO, "skipping directory %s\n", fname);
 			continue;
 		}
@@ -943,31 +989,37 @@ struct file_list *send_file_list(int f, int argc, char *argv[])
 					dir = fname;
 				fname = p + 1;
 			}
-		} else if (f != -1 && (p = strrchr(fname, '/'))) {
+		} else if (f != -1 && implied_dirs && (p=strrchr(fname,'/')) && p != fname) {
 			/* this ensures we send the intermediate directories,
 			   thus getting their permissions right */
+			char *lp = lastpath, *fn = fname, *slash = fname;
 			*p = 0;
-			if (strcmp(lastpath, fname)) {
-				strlcpy(lastpath, fname, sizeof(lastpath));
-				*p = '/';
-				for (p = fname + 1; (p = strchr(p, '/'));
-				     p++) {
-					int copy_links_saved = copy_links;
-					int recurse_saved = recurse;
-					*p = 0;
-					copy_links = copy_unsafe_links;
-					/* set recurse to 1 to prevent make_file
-					   from ignoring directory, but still
-					   turn off the recursive parameter to
-					   send_file_name */
-					recurse = 1;
-					send_file_name(f, flist, fname, 0,
-						       0);
-					copy_links = copy_links_saved;
-					recurse = recurse_saved;
-					*p = '/';
+			/* Skip any initial directories in our path that we
+			 * have in common with lastpath. */
+			while (*fn && *lp == *fn) {
+				if (*fn == '/')
+					slash = fn;
+				lp++, fn++;
+			}
+			*p = '/';
+			if (fn != p || (*lp && *lp != '/')) {
+				int copy_links_saved = copy_links;
+				int recurse_saved = recurse;
+				copy_links = copy_unsafe_links;
+				/* set recurse to 1 to prevent make_file
+				 * from ignoring directory, but still
+				 * turn off the recursive parameter to
+				 * send_file_name */
+				recurse = 1;
+				while ((slash = strchr(slash+1, '/')) != 0) {
+					*slash = 0;
+					send_file_name(f, flist, fname, 0, 0);
+					*slash = '/';
 				}
-			} else {
+				copy_links = copy_links_saved;
+				recurse = recurse_saved;
+				*p = 0;
+				strlcpy(lastpath, fname, sizeof lastpath);
 				*p = '/';
 			}
 		}
@@ -979,9 +1031,9 @@ struct file_list *send_file_list(int f, int argc, char *argv[])
 			olddir = push_dir(dir, 1);
 
 			if (!olddir) {
-				io_error = 1;
-				rprintf(FERROR, "push_dir %s : %s\n",
-					dir, strerror(errno));
+				io_error |= IOERR_GENERAL;
+				rprintf(FERROR, "push_dir %s failed: %s\n",
+					full_fname(dir), strerror(errno));
 				continue;
 			}
 
@@ -996,8 +1048,8 @@ struct file_list *send_file_list(int f, int argc, char *argv[])
 		if (olddir != NULL) {
 			flist_dir = NULL;
 			if (pop_dir(olddir) != 0) {
-				rprintf(FERROR, "pop_dir %s : %s\n",
-					dir, strerror(errno));
+				rprintf(FERROR, "pop_dir %s failed: %s\n",
+					full_fname(dir), strerror(errno));
 				exit_cleanup(RERR_FILESELECT);
 			}
 		}
@@ -1011,16 +1063,16 @@ struct file_list *send_file_list(int f, int argc, char *argv[])
 		finish_filelist_progress(flist);
 	}
 
-	clean_flist(flist, 0);
+	clean_flist(flist, 0, 0);
 
 	/* now send the uid/gid list. This was introduced in protocol
 	   version 15 */
-	if (f != -1 && remote_version >= 15) {
+	if (f != -1) {
 		send_uid_list(f);
 	}
 
-	/* if protocol version is >= 17 then send the io_error flag */
-	if (f != -1 && remote_version >= 17) {
+	/* send the io_error flag */
+	if (f != -1) {
 		extern int module_id;
 		write_int(f, lp_ignore_errors(module_id) ? 0 : io_error);
 	}
@@ -1065,7 +1117,7 @@ struct file_list *recv_file_list(int f)
 
 	for (flags = read_byte(f); flags; flags = read_byte(f)) {
 		int i = flist->count;
-		
+
 		flist_expand(flist);
 
 		receive_file_entry(&flist->files[i], flags, f);
@@ -1086,19 +1138,19 @@ struct file_list *recv_file_list(int f)
 	if (verbose > 2)
 		rprintf(FINFO, "received %d names\n", flist->count);
 
-	clean_flist(flist, relative_paths);
+	clean_flist(flist, relative_paths, 1);
 
 	if (show_filelist_p()) {
 		finish_filelist_progress(flist);
 	}
 
 	/* now recv the uid/gid list. This was introduced in protocol version 15 */
-	if (f != -1 && remote_version >= 15) {
+	if (f != -1) {
 		recv_uid_list(f, flist);
 	}
 
-	/* if protocol version is >= 17 then recv the io_error flag */
-	if (f != -1 && remote_version >= 17 && !read_batch) {	/* dw-added readbatch */
+	/* recv the io_error flag */
+	if (f != -1 && !read_batch) {	/* dw-added readbatch */
 		extern int module_id;
 		extern int ignore_errors;
 		if (lp_ignore_errors(module_id) || ignore_errors) {
@@ -1243,11 +1295,11 @@ void flist_free(struct file_list *flist)
 
 /*
  * This routine ensures we don't have any duplicate names in our file list.
- * duplicate names can cause corruption because of the pipelining 
+ * duplicate names can cause corruption because of the pipelining
  */
-static void clean_flist(struct file_list *flist, int strip_root)
+static void clean_flist(struct file_list *flist, int strip_root, int no_dups)
 {
-	int i;
+	int i, prev_i = 0;
 	char *name, *prev_name = NULL;
 
 	if (!flist || flist->count == 0)
@@ -1256,8 +1308,9 @@ static void clean_flist(struct file_list *flist, int strip_root)
 	qsort(flist->files, flist->count,
 	      sizeof(flist->files[0]), (int (*)()) file_compare);
 
-	for (i = 0; i < flist->count; i++) {
+	for (i = no_dups? 0 : flist->count; i < flist->count; i++) {
 		if (flist->files[i]->basename) {
+			prev_i = i;
 			prev_name = f_name(flist->files[i]);
 			break;
 		}
@@ -1272,6 +1325,11 @@ static void clean_flist(struct file_list *flist, int strip_root)
 					"removing duplicate name %s from file list %d\n",
 					name, i);
 			}
+			/* Make sure that if we unduplicate '.', that we don't
+			 * lose track of a user-specified starting point (or
+			 * else deletions will mysteriously fail with -R). */
+			if (flist->files[i]->flags & FLAG_DELETE)
+				flist->files[prev_i]->flags |= FLAG_DELETE;
 			/* it's not great that the flist knows the semantics of
 			 * the file memory usage, but i'd rather not add a flag
 			 * byte to that struct.
@@ -1281,6 +1339,10 @@ static void clean_flist(struct file_list *flist, int strip_root)
 			else
 				free_file(flist->files[i]);
 		}
+		else
+			prev_i = i;
+		/* We set prev_name every iteration to avoid it becoming
+		 * invalid when names[][] in f_name() wraps around. */
 		prev_name = name;
 	}
 
